<?php

/**
 * return form
 */
function resource_booking_widget_settings_form($field) {
    //provide a selection for choosing node type that represents a resource
  $node_types = node_get_types('types');
  foreach ($node_types as $key => $node_spec) {
    $options[$key] = $node_spec->name;
  }
  
  $form['resource_booking_node'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#required' => TRUE,
    '#title' => t('Select node type'),
    '#description' => t('Select node type that has bookable resources'),
    '#default_value' => isset($field['resource_booking_node'])? $field['resource_booking_node'] : '',
  );
  $form['resource_schedule_url'] = array(
    '#type' => 'textfield',
    '#title' => t('Path for resource schedule'),
    '#description' => t('Path to the calendar view that displays the resource schedule. Do not enter month/day/year in the url.'),
    '#default_value' => empty($field['resource_schedule_url'])? 'rs-calendar' : $field['resource_schedule_url'],
    '#field_suffix' => '/',
    '#required' => TRUE,
    '#size' => 16,
  );
  //allow for selection of resource types
  $query = <<<SQL
    SELECT t.tid, t.name, v.name v_name FROM {term_data} t
    INNER JOIN {vocabulary} v on v.vid= t.vid
    ORDER BY v.name, t.name
SQL;
  $result = db_query($query);
  while ($record = db_fetch_object($result)) {
    $resource_terms[$record->v_name][$record->tid] = $record->name;
  }
  $form['resource_terms'] = array(
    '#type' => 'select',
    '#title' => t('Select allowed resource types'),
    '#options' => $resource_terms,
    '#multiple' => TRUE,
    '#default_value' => empty($field['resource_terms'])? array() : $field['resource_terms'],
    '#description' => t('Select allowed resource types. To select all resource types and new ones added in future, do not select anything.'),
  );
  $form['resource_terms_multiple'] = array(
    '#type' => 'select',
    '#title' => t('Allow multiple'),
    '#options' => $resource_terms,
    '#multiple' => TRUE,
    '#default_value' => empty($field['resource_terms_multiple'])? array() : $field['resource_terms_multiple'],
    '#description' => t('Select allowed resource types that allow multiple resources to be booked.'),
  );
  $form['resource_terms_duplicate'] = array(
    '#type' => 'select',
    '#title' => t('Allow duplicate'),
    '#options' => $resource_terms,
    '#multiple' => TRUE,
    '#default_value' => empty($field['resource_terms_duplicate'])? array() : $field['resource_terms_duplicate'],
    '#description' => t('Select allowed resource types that allowe booking of booked resources.'),
  );
  //check if the resource has date fields. use that
  $content_info = _content_type_info();
  $type_name = arg(3);
  $ct_fields = array(t('None'));
  if (!is_array($content_info['content types'][$type_name]) && !empty($field['resource_booking_ct_date'])) {
    //type name not found by url. means it's probably content export. so find the content type
    //based on the content type that has the date field that is chosen.
    //note if the date field is in another content type as well then the list of fields will be wrong
    //but functionality should not be affected.
    foreach ($node_types as $key => $node_spec) {
      if (isset($content_info['content types'][$key]['fields'][$field['resource_booking_ct_date']])){
        $type_name = $key;
        break;
      }
    }
  }

  if (is_array($content_info['content types'][$type_name])) {
    foreach ($content_info['content types'][$type_name]['fields'] as $field_name => $ctfield) {
      $ct_fields[$field_name] = $ctfield['widget']['label'];
    }
  }
  $form['resource_booking_ct_date'] = array(
    '#type' => 'select',
    '#title' => t('Date field'),
    '#options' => $ct_fields,
    '#default_value' => empty($field['resource_booking_ct_date'])? 0 : $field['resource_booking_ct_date'],
    '#description' => t('If the content type already has a date field, then select that field to use date values for resource booking.'),
  );
  
  return $form;
}

/**
 * Save function
 */
function resource_booking_widget_settings_save($field) {
  return array('resource_booking_node', 'resource_schedule_url', 'resource_terms',
    'resource_terms_multiple', 'resource_terms_duplicate', 'resource_booking_ct_date');  
}

/**
 * render the element
 */
function resource_booking_process_widget($element, $edit, $form_state, $form) {
  ahah_helper_register($form, $form_state);
  $field_name = $element['#field_name'];
  $field = $form['#field_info'][$field_name];
  $defaults = isset($element['#value']) ? $element['#value'] : array();
  $resources = array();
  $rb_schedule_options = array('attributes' => array('target' => '_blank'));
  $sdate = NULL;
  $edate = NULL;
  $rb_status = NULL;
  if (isset($defaults['resource_booking']['sdate'])) {
    //choose the current selection
    if (is_array($defaults['resource_booking']['sdate'])) {
      $cdate['#date_format'] = variable_get('date_format_short', 'm/d/Y - H:i');
      $cdate['#date_timezone'] = date_default_timezone_name();
      //start date
      $cdate['#value'] = $defaults['resource_booking']['sdate'];
      $sdate =  strtotime(date_popup_input_value($cdate));
      //end date
      $cdate['#value'] = $defaults['resource_booking']['edate'];
      $edate =  strtotime(date_popup_input_value($cdate));
    }
    else {
      $sdate = strtotime($defaults['resource_booking']['sdate']);
      $edate = strtotime($defaults['resource_booking']['edate']);
    }
    foreach ($defaults as $default) {
      $resources[] = $default['rnid'];
      $rb_status[$default['rnid']] = $default['rb_status'];
    }
  }
  elseif (isset($defaults[0]['sdate'])){
    $sdate = $defaults[0]['sdate'];
    $edate = $defaults[0]['edate'];
    foreach ($defaults as $default) {
      $resources[] = $default['rnid'];
      $rb_status[$default['rnid']] = $default['rb_status'];
    }
  }
  else {  //if start and end are not set, then default it to today
    $today_9am = mktime(9, 0, 0, date('m'), date('d'), date('Y'));
    $sdate = $today_9am;
    $edate = strtotime('+1 day', $sdate);
  }

  $element['resource_booking'] = array(
    '#type' => 'fieldset',
    '#title' => t('Resources'),
    '#theme' => 'resource_booking_widget_form',
    '#prefix' => '<div id="rb_status-wrapper">',
    '#suffix' => '</div>',
    '#tree'   => TRUE,
  );

  if ($field['widget']['resource_booking_ct_date']) {
    //use date fields from content. so hide the rb date fields
    $cls = 'hidden';
    $path = drupal_get_path('module', 'resource_booking');
    drupal_add_css($path . '/resource_booking.css');
    drupal_add_js($path . '/resource_booking.js');
    $jfields = array(
      'resource_booking' => array(
        'date_field' => $field['widget']['resource_booking_ct_date'],
        'rb_field' => $field_name,
      )
    );
    drupal_add_js($jfields, 'setting');
    $content_info = _content_type_info();
    $type_name = $form['#node']->type;
    $date_format = $content_info['content types'][$type_name]['fields'][$field['widget']['resource_booking_ct_date']]['widget']['input_format'];
  }
  else {
    $cls = '';
    $date_format = variable_get('date_format_short', 'm/d/Y - H:i');
  }
  $element['resource_booking']['sdate'] = array(
    '#type' => 'date_popup',
    '#date_type' => DATE_UNIX,
    '#title' => t('Start date and time'),
    '#default_value' => date('Y-m-d H:i', $sdate),
    '#prefix' => '<div class="hidden resource-booking-start-date">', '#suffix' => '</div>',
    '#date_format' => $date_format,
  );
  $element['resource_booking']['edate'] = array(
    '#type' => 'date_popup',
    '#date_type' => DATE_UNIX,
    '#title' => t('End date and time'),
    '#default_value' => date('Y-m-d H:i', $edate),
    '#prefix' => '<div class="hidden resource-booking-end-date">', '#suffix' => '</div>',
    '#date_format' => $date_format,
  );
  $element['resource_booking']['btn_check_status'] = array(
    '#type' => 'button',
    '#value' => t('Refresh availability'),
    '#ahah' => array(
      'event'   => 'click',
      'path'    => ahah_helper_path(array($field_name)),
      'wrapper' => 'rb_status-wrapper',
    ),
  );
  //dummy item to prevent jumping on ajax call
  $element['resource_booking']['dmy'] = array(
    '#type' => 'hidden',
    '#value' => '',
    '#prefix' => '<div style="clear:both;">',
    '#suffix' => '</div>',
  );

  //choose 1 minute higher than the entered value to avoid end of one with start of another
  $start_date = strtotime("+1 minute", $sdate);
  //choose 1 minute less than the entered value to avoid start of one with end of another
  $end_date = strtotime("-1 minute", $edate);
  if (!empty($start_date) && !empty($end_date)) {
    $show_availability = TRUE;
  }
  $allow_multiple = empty($field['widget']['resource_terms_multiple'])? array() : $field['widget']['resource_terms_multiple'];
  $allow_duplicate  = empty($field['widget']['resource_terms_duplicate'])? array() : $field['widget']['resource_terms_duplicate'];
  
  $data = _resource_booking_get_availability_data($start_date, $end_date, $field);
  extract($data);

  $cnt = 0;
  foreach ($options as $tid => $term_resources) {
    foreach ($term_resources as $name => $options_values) {
      $options_values[0] = t('None');
      $default_values = array_keys($options_values);
      //get the value that's relevant for this resource
      $result = array_intersect($default_values, $resources);
      if (in_array($tid, $allow_multiple)) {
        $select_type = 'checkboxes';
        $default_value = $result;
        unset($options_values[0]);
      }
      else {
        $select_type = 'radios';
        $default_value = array_pop($result);
      }
      $aduplicate = in_array($tid, $allow_duplicate);
      $element['resource_booking']['resources']["fs-$cnt"] = array(
        '#type' => 'fieldset',
        '#title' => check_plain($name),
        '#collapsible' => TRUE,
      );
      $element['resource_booking']['resources']["fs-$cnt"]["nid-$cnt"]= array(
        '#type' => $select_type,
        '#options' => $options_values,
        '#default_value' => $default_value,
        '#allow_duplicates' => $aduplicate,
      );
      if (user_access(CONST_RESOURCE_BOOKING_CHANGE_STATUS)) {
        $rb_status_enabled = 'enabled';
      }
      else {
        $rb_status_enabled = 'disabled';
      }
      foreach ($options_values as $key => $value) {
        if (!empty($key)) {
          $element['resource_booking']['resources']["fs-$cnt"]["rb_status-$cnt"][$key] = array(
            '#type' => 'select',
            '#options' => _resource_booking_field_get_rb_status(),
            '#default_value' => (isset($rb_status) && !empty($rb_status[$key])) ? $rb_status[$key] : CONST_RESOURCE_BOOKING_REQUESTED,
            '#attributes' => array($rb_status_enabled => $rb_status_enabled),
          );
          if (!$show_availability) {
            $available = t('Unknown');
          }
          else {
            $available = empty($availability[$key]['rb_status'])? t('Available') : t('Booked');
          }

          //provide a link to the resouce booking schedule
          if (user_access(CONST_RESOURCE_BOOKING_VIEW_RESOURCE_SCHEDULE)) {
            $available_link = l($available, $field['widget']['resource_schedule_url'] . '/month/' . date('Y', $start_date) . '-' . date('m', $start_date) . '/' . $key, $rb_schedule_options);
          }


          $element['resource_booking']['resources']["fs-$cnt"]["availability-$cnt"][$key] = array(
            '#type' => 'markup',
            '#markup' => $available_link,
            '#value' => $available,
          );
        }
        else {
          //"none" row
          $element['resource_booking']['resources']["fs-$cnt"]["rb_status-$cnt"][$key] = array(
            '#markup' => '',
          );
          $element['resource_booking']['resources']["fs-$cnt"]["availability-$cnt"][$key] = array(
            '#type' => 'markup',
            '#markup' => '',
          );
        }
      }

      $cnt++;
    }
  }
  return $element;
}

function _resource_booking_get_availability_data($start_date, $end_date, $field) {
  $join = '';
  $fields = '';
  $args = array();
  if (!empty($start_date) && !empty($end_date)) {
    $join = "LEFT OUTER JOIN {resource_booking} rb on rb.rnid = n.nid AND " .
    "rb_status != '" . CONST_RESOURCE_BOOKING_DECLINED . "'" .
    " AND ( " .
      "(%d >= sdate AND %d <= edate) OR " .
      "(%d >= sdate AND %d <= edate) OR " .
      "(sdate >= %d AND sdate <= %d) OR " .
      "(edate >= %d AND edate <= %d)" .
      ")";
    $fields = ', rnid, rb_status';
    $args[] = $start_date;
    $args[] = $start_date;
    $args[] = $end_date;
    $args[] = $end_date;
    $args[] = $start_date;
    $args[] = $end_date;
    $args[] = $start_date;
    $args[] = $end_date;
  }
  $query = "SELECT n.nid, n.vid, n.title, td.name, td.tid $fields 
  FROM {node} n
  INNER JOIN {term_node} tn on tn.nid = n.nid and tn.vid = n.vid
  INNER JOIN {term_data} td on td.tid = tn.tid
  $join";
  
  //only published nodes
  $where[] = ' n.status = 1 ';
  //filter for only selected resource node type
  $where[] = " n.type = '" . $field['widget']['resource_booking_node'] . "' ";
  //if terms are selected filter for that
  if (!empty($field['widget']['resource_terms'])) {
    $where[] = ' td.tid in (' . implode(',', $field['widget']['resource_terms']) . ') ';
  }
  $query .= ' WHERE ' . implode(' AND ', $where);
  $result = db_query($query, $args);
  $availability = array();
  $options = array();
  while ($record = db_fetch_object($result)) {
    $options[$record->tid][$record->name][$record->nid] = $record->title;
    if (!empty($record->nid)) {
      $availability[$record->nid] = array(
        'nid' => $record->nid,
        'vid' => $record->vid,
        'rb_status' => $record->rb_status, 
      );
    }
  }
  return compact('options', 'availability');
}
